# coding=utf-8
import json


class OkexAPIException(Exception):

    def __init__(self, response):
        print(response.text + ', ' + str(response.status_code))
        self.code = 0
        try:
            json_res = response.json()
        except ValueError:
            self.message = 'Invalid JSON error message from Okex: {}'.format(response.text)
        else:
            if "error_code" in json_res.keys() and "error_message" in json_res.keys():
                self.code = json_res['error_code']
                self.message = json_res['error_message']
            else:
                self.code = 'None'
                self.message = 'System error'

        self.status_code = response.status_code
        self.response = response
        self.request = getattr(response, 'request', None)

    def __str__(self):  # pragma: no cover
        return 'API Request Error(error_code=%s): %s' % (self.code, self.message)


class OkexRequestException(Exception):

    def __init__(self, message):
        self.message = message

    def __str__(self):
        return 'OkexRequestException: %s' % self.message


class OkexParamsException(Exception):

    def __init__(self, message):
        self.message = message

    def __str__(self):
        return 'OkexParamsException: %s' % self.message



from notecoin.okex.websocket.channel import PrivateChannel
from notecoin.okex.websocket.connect import PrivateConnect
from notetool.secret import read_secret

api_key = read_secret(cate1='coin', cate2='okex', cate3='api_key')
secret_key = read_secret(cate1='coin', cate2='okex', cate3='secret_key')
passphrase = read_secret(cate1='coin', cate2='okex', cate3='passphrase')

connect = PrivateConnect(channels=[PrivateChannel.private_positions(instType='ANY', uly=None, instId=None)],
                         api_key=api_key,
                         secret_key=secret_key,
                         passphrase=passphrase)
connect.run()



import base64
import hmac
import json
import logging
import time

from notecoin.okex.websocket.utils import (check, get_local_timestamp, partial,
                                           update_asks, update_bids)

from websocket import WebSocket, WebSocketException, create_connection


class BaseConnect:
    def __init__(self, url, channels, api_key=None, secret_key=None, passphrase=None):
        self.url = url
        self.channels = channels
        self.api_key = api_key
        self.secret_key = secret_key
        self.passphrase = passphrase

        self.ws: WebSocket = create_connection(self.url)
        self.subscribe_start()

    def run(self):
        while True:
            try:
                res = self.ws.recv()
            except (TimeoutError, WebSocketException):
                try:
                    self.ping()
                except Exception as e:
                    logging.warning(f"连接关闭，正在重连:{e}")
                    self.subscribe_restart()
                continue
            self.handle_data(res)

    def handle_data(self, res):
        pass

    def ping(self):
        self.ws.send('ping')
        res = self.ws.recv()
        print(res)
        assert res == 'pong'

    def subscribe_restart(self):
        self.subscribe_stop()
        self.subscribe_start()

    def subscribe_start(self):
        self.ws: WebSocket = create_connection(self.url)
        sub_param = {"op": "subscribe", "args": self.channels}
        sub_str = json.dumps(sub_param)
        self.ws.send(sub_str)
        print(f"send: {sub_str}")
        res = self.ws.recv()
        print(f"recv: {res}")
        time.sleep(1)

    def subscribe_stop(self):
        self.ws: WebSocket = create_connection(self.url)
        sub_param = {"op": "unsubscribe", "args": self.channels}
        sub_str = json.dumps(sub_param)
        self.ws.send(sub_str)
        print(f"send: {sub_str}")
        res = self.ws.recv()
        print(f"recv: {res}")


class PublicConnect(BaseConnect):
    def __init__(self, channels, *args, **kwargs):
        super(PublicConnect, self).__init__(url="wss://ws.okx.com:8443/ws/v5/public",
                                            channels=channels, *args, **kwargs)

    def handle_data(self, res):
        res = eval(res)
        print(f"{get_local_timestamp()}\t{res}")
        if 'event' in res:
            return
        l = []
        for i in res['arg']:
            if 'books' in res['arg'][i] and 'books5' not in res['arg'][i]:
                # 订阅频道是深度频道
                if res['action'] == 'snapshot':
                    for m in l:
                        if res['arg']['instId'] == m['instrument_id']:
                            l.remove(m)
                    # 获取首次全量深度数据
                    bids_p, asks_p, instrument_id = partial(res)
                    d = {}
                    d['instrument_id'] = instrument_id
                    d['bids_p'] = bids_p
                    d['asks_p'] = asks_p
                    l.append(d)

                    # 校验checksum
                    checksum = res['data'][0]['checksum']
                    # print('推送数据的checksum为:' + str(checksum))
                    check_num = check(bids_p, asks_p)
                    # print('校验后的checksum为:' + str(check_num))
                    if check_num == checksum:
                        print("校验结果为:True")
                    else:
                        print("校验结果为:False，正在重新订阅……")
                        self.subscribe_stop()
                        self.subscribe_start()

                elif res['action'] == 'update':
                    for j in l:
                        if res['arg']['instId'] == j['instrument_id']:
                            # 获取全量数据
                            bids_p = j['bids_p']
                            asks_p = j['asks_p']
                            # 获取合并后数据
                            bids_p = update_bids(res, bids_p)
                            asks_p = update_asks(res, asks_p)

                            # 校验checksum
                            checksum = res['data'][0]['checksum']
                            # print('推送数据的checksum为:' + str(checksum))
                            check_num = check(bids_p, asks_p)
                            # print('校验后的checksum为:' + str(check_num))
                            if check_num == checksum:
                                print("校验结果为:True")
                            else:
                                print("校验结果为:False，正在重新订阅……")
                                self.subscribe_stop()
                                self.subscribe_start()


class PrivateConnect(BaseConnect):

    def __init__(self, channels, *args, **kwargs):
        super(PrivateConnect, self).__init__(url='wss://ws.okx.com:8443/ws/v5/private',
                                             channels=channels, *args, **kwargs)

    def handle_data(self, res):
        print(res)

    def subscribe_start(self):
        self.ws = create_connection(self.url)
        timestamp = str(get_local_timestamp())
        message = timestamp + 'GET' + '/users/self/verify'
        mac = hmac.new(bytes(self.secret_key, encoding='utf8'), bytes(message, encoding='utf-8'), digestmod='sha256')
        sign = base64.b64encode(mac.digest()).decode("utf-8")
        login_param = {"op": "login", "args": [{"apiKey": self.api_key,
                                                "passphrase": self.passphrase,
                                                "timestamp": timestamp,
                                                "sign": sign}]}
        login_str = json.dumps(login_param)
        self.ws.send(login_str)
        print(f"send: {login_str}")


class TradeConnect(PrivateConnect):
    def __init__(self, *args, **kwargs):
        super(TradeConnect, self).__init__(*args, **kwargs)

    def handle_data(self, res):
        print(res)



from notecoin.huobi.connection import (RestApiSyncClient, SubscribeClient,
                                       WebSocketReqClient)
from notecoin.huobi.constant import HttpMethod


class GenericClient(object):

    def __init__(self, *args, **kwargs):
        """
        Create the request client instance.
        :param kwargs: The option of request connection.
            api_key: The public key applied from Huobi.
            secret_key: The private key applied from Huobi.
            url: The URL name like "https://api.huobi.pro".
            init_log: to init logger
        """
        self.__kwargs = kwargs
        self.rest_api_sync_client = RestApiSyncClient(*args, **kwargs)
        self.web_socket_req_client = WebSocketReqClient(*args, **kwargs)
        self.sub_socket_req_client = SubscribeClient(*args, **kwargs)

    def get_exchange_timestamp(self) -> int:
        """
        Get the timestamp from Huobi server. The timestamp is the Unix timestamp in millisecond.
        The count shows how many milliseconds passed from Jan 1st 1970, 00:00:00.000 at UTC.
        e.g. 1546300800000 is Thu, 1st Jan 2019 00:00:00.000 UTC.

        :return: The timestamp in UTC
        """
        channel = "/v1/common/timestamp"
        params = {}

        return self.rest_api_sync_client.request_process(HttpMethod.GET, channel, params)

    def get_exchange_currencies(self):
        """
        Get all the trading assets and currencies supported in huobi.
        The information of trading instrument, including base currency, quote precision, etc.

        :return: The information of trading currencies.
        """
        channel = "/v1/common/currencys"
        params = {}

        return self.rest_api_sync_client.request_process(HttpMethod.GET, channel, params)

    def get_exchange_symbols(self):
        """
        Get all the trading assets and currencies supported in huobi.
        The information of trading instrument etc.

        :return: The information of trading instrument.
        """
        channel = "/v1/common/symbols"
        params = {}

        return self.rest_api_sync_client.request_process(HttpMethod.GET, channel, params)

    def get_exchange_info(self):
        """
        Get all the trading assets and currencies supported in huobi.
        The information of trading instrument, including base currency, quote precision, etc.

        :return: The information of trading instrument and currencies.
        """

        ret = {
            "symbol_list": self.get_exchange_symbols(),
            "currencies": self.get_exchange_currencies()
        }
        return ret

    def get_reference_currencies(self, currency: 'str' = None, is_authorized_user: 'bool' = None) -> list:
        """
        Get all the trading assets and currencies supported in huobi.
        The information of trading instrument, including base currency, quote precision, etc.

        :param currency: btc, ltc, bch, eth, etc ...(available currencies in Huobi Global)
        :param is_authorized_user: is Authorized user? True or False
        :return: The information of trading instrument and currencies.
        """
        channel = "/v2/reference/currencies"
        params = {
            "currency": currency,
            "authorizedUser": is_authorized_user
        }

        return self.rest_api_sync_client.request_process(HttpMethod.GET, channel, params)

    def get_system_status(self) -> str:
        """
        get system status

        :return: system status.
        """
        channel = "/api/v2/summary.json"
        temp = self.rest_api_sync_client.__server_url
        self.rest_api_sync_client.__server_url = "https://status.huobigroup.com"
        res = self.rest_api_sync_client.request_process(HttpMethod.GET, channel, {})
        self.rest_api_sync_client.__server_url = temp
        return res

    def get_market_status(self):
        channel = "/v2/market-status"

        return self.rest_api_sync_client.request_process(HttpMethod.GET, channel, {})



import os

from notedrive.tables.core import SqliteTable
from notetool import read_secret


class KlineDetail(SqliteTable):
    def __init__(self, table_name='symbol_info', db_path=None, *args, **kwargs):
        if db_path is None:
            db_path = os.path.abspath(
                os.path.dirname(__file__)) + '/data/coin.db'

        super(KlineDetail, self).__init__(db_path=db_path, table_name=table_name,                                          *args, **kwargs)

        self.columns = ['coid_id', 'ts', 'count', 'open', 'close', 'low', 'high', 'vol', 'amount']

    def create(self):
        self.execute("""
                create table if not exists `{}` (
                `coid_id`     VARCHAR(20)	--COMMENT '交易对'
                ,`ts`         BIGINT(32)  --COMMENT '调整新加坡时间的时间戳'
                ,`count`      INT(32)     --COMMENT '交易次数'
                ,`open`       FLOAT   --COMMENT '本阶段开盘价'
                ,`close`      float   --COMMENT '本阶段收盘价'
                ,`low`	      float   --COMMENT '本阶段最低价'
                ,`high`	      float   --COMMENT '本阶段最高价'
                ,`vol`        float   --COMMENT '以报价币种计量的交易量'
                ,`amount`     float   --COMMENT '以基础币种计量的交易量'
                ,primary key (`coid_id`,`ts`)           
                      );
                    """.format(self.table_name))

    def select_all(self):
        """
        返回全表数据
        """
        return self.select("select * from table_name")

    def select(self, sql=None, condition: dict = None):
        """
        根据sql或者指定条件选择数据
        :param sql: sql
        :param condition: 条件
        :return: 记录list
        """
        if sql is None:
            equal2 = self._condition2equal(condition)
            sql = """select * from {} where {}""".format(self.table_name, ' and '.join(equal2))
        else:
            sql = self.sql_format(sql)

        rows = self.execute(sql)
        print(rows)
        return [] if rows is None else [dict(zip(self.columns, row)) for row in rows]



from fastapi import FastAPI
from notecoin.okex.server.const import account_account, market_tickers
from 


seller = AutoSeller()


app = FastAPI()
app.include_router(account_account)
app.include_router(market_tickers)

# uvicorn notecoin:app --host '0.0.0.0' --port 8444 --reload
